import * as React from 'react';
import * as ReactDOM from 'react-dom';
export const createPortalNode = () => {
    let initialProps = {};
    return Object.assign(document.createElement('div'), {
        setPortalProps: (props) => {
            initialProps = props;
        },
        getInitialPortalProps: () => {
            return initialProps;
        }
    });
};
export class InPortal extends React.Component {
    constructor(props) {
        super(props);
        this.addPropsChannel = () => {
            Object.assign(this.props.node, {
                setPortalProps: (props) => {
                    // Rerender the child node here if/when the out portal props change
                    this.setState({ nodeProps: props });
                }
            });
        };
        this.state = {
            nodeProps: this.props.node.getInitialPortalProps()
        };
    }
    componentDidMount() {
        this.addPropsChannel();
    }
    componentDidUpdate() {
        this.addPropsChannel();
    }
    render() {
        const { children, node } = this.props;
        return ReactDOM.createPortal(React.Children.map(children, (child) => {
            if (!React.isValidElement(child))
                return child;
            return React.cloneElement(child, this.state.nodeProps);
        }), node);
    }
}
export class OutPortal extends React.Component {
    constructor(props) {
        super(props);
        this.placeholderNode = React.createRef();
        this.passPropsThroughPortal();
    }
    passPropsThroughPortal() {
        const propsForTarget = Object.assign({}, this.props, { node: undefined });
        this.props.node.setPortalProps(propsForTarget);
    }
    componentDidMount() {
        const placeholder = this.placeholderNode.current;
        placeholder.parentNode.replaceChild(this.props.node, placeholder);
        this.passPropsThroughPortal();
    }
    componentDidUpdate() {
        this.passPropsThroughPortal();
    }
    componentWillUnmount() {
        const { node } = this.props;
        node.parentNode.removeChild(node);
        this.props.node.setPortalProps({});
    }
    render() {
        // Render a placeholder to the DOM, so we can get a reference into
        // our location in the DOM, and swap it out for the portaled node.
        return React.createElement("div", { ref: this.placeholderNode });
    }
}
//# sourceMappingURL=index.js.map